#include "drvinst.h"

//
// Function:  GetKeyValue
//
// Purpose:   Retrieve the value of a key from the inf file.
//
// Arguments:
//    hInf        [in]  Inf file handle.
//    lpszSection [in]  Section name.
//    lpszKey     [in]  Key name.
//    dwIndex     [in]  Key index.
//    lppszValue  [out] Key value.
//
// Returns:   S_OK on success, otherwise and error code.
//
// Notes:
//

HRESULT
GetKeyValue (
    HINF hInf,
    __in LPCWSTR lpszSection,
    __in_opt LPCWSTR lpszKey,
    DWORD  dwIndex,
    __deref_out LPWSTR *lppszValue)
{
    INFCONTEXT  infCtx;
    __range(0, 512) DWORD       dwSizeNeeded;
    HRESULT     hr;

    *lppszValue = NULL;

    if ( SetupFindFirstLineW(hInf,
                             lpszSection,
                             lpszKey,
                             &infCtx) == FALSE )
    {
        return HRESULT_FROM_WIN32(GetLastError());
    }

    if ( SetupGetStringFieldW(&infCtx,
                              dwIndex,
                              NULL,
                              0,
                              &dwSizeNeeded) )
    {
        *lppszValue = (LPWSTR)CoTaskMemAlloc( sizeof(WCHAR) * dwSizeNeeded );

        if ( !*lppszValue  )
        {
        return HRESULT_FROM_WIN32(ERROR_NOT_ENOUGH_MEMORY);
        }

        if ( SetupGetStringFieldW(&infCtx,
                                dwIndex,
                                *lppszValue,
                                dwSizeNeeded,
                                NULL) == FALSE )
        {

            hr = HRESULT_FROM_WIN32(GetLastError());

            CoTaskMemFree( *lppszValue );
            *lppszValue = NULL;
        }
        else
        {
            hr = S_OK;
        }
    }
    else
    {
        DWORD dwErr = GetLastError();
        hr = HRESULT_FROM_WIN32(dwErr);
    }

    return hr;
}

//
// Function:  GetPnpID
//
// Purpose:   Retrieve PnpID from an inf file.
//
// Arguments:
//    lpszInfFile [in]  Inf file to search.
//    lppszPnpID  [out] PnpID found.
//
// Returns:   TRUE on success.
//
// Notes:
//

HRESULT
GetPnpID (
    __in LPWSTR lpszInfFile,
    __deref_out LPWSTR *lppszPnpID)
{
    HINF    hInf;
    LPWSTR  lpszModelSection;
    HRESULT hr;

    *lppszPnpID = NULL;

    hInf = SetupOpenInfFileW( lpszInfFile,
                              NULL,
                              INF_STYLE_WIN4,
                              NULL );

    if ( hInf == INVALID_HANDLE_VALUE )
    {

        return HRESULT_FROM_WIN32(GetLastError());
    }

    //
    // Read the Model section name from Manufacturer section.
    //

    hr = GetKeyValue( hInf,
                      L"Manufacturer",
                      NULL,
                      1,
                      &lpszModelSection );

    if ( hr == S_OK )
    {

        //
        // Read PnpID from the Model section.
        //

        hr = GetKeyValue( hInf,
                          lpszModelSection,
                          NULL,
                          2,
                          lppszPnpID );

        CoTaskMemFree( lpszModelSection );
    }

    SetupCloseInfFile( hInf );

    return hr;
}

//
// Function:  ErrMsg
//
// Purpose:   Insert text for each network component type.
//
// Arguments:
//    hr  [in]  Error code.
//
// Returns:   None.
//
// Notes:
//

VOID ErrMsg (HRESULT hr,
             LPCWSTR  lpFmt,
             ...)
{

    LPWSTR   lpSysMsg;
    WCHAR    buf[400];
    ULONG    offset;
    va_list  vArgList;


    if ( hr != 0 ) {
        StringCchPrintfW ( buf,
            celems(buf),
            L"Error %#lx: ",
            hr );
    }
    else {
        buf[0] = 0;
    }

    offset = wcslen( buf );

    va_start( vArgList,
              lpFmt );
    StringCchVPrintfW ( buf+offset,
        celems(buf) - offset,
        lpFmt,
        vArgList );

    va_end( vArgList );

    if ( hr != 0 ) {
        FormatMessageW( FORMAT_MESSAGE_ALLOCATE_BUFFER |
                       FORMAT_MESSAGE_FROM_SYSTEM |
                       FORMAT_MESSAGE_IGNORE_INSERTS,
                       NULL,
                       hr,
                       MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                       (LPWSTR)&lpSysMsg,
                       0,
                       NULL );
        if ( lpSysMsg ) {

            offset = wcslen( buf );

            StringCchPrintfW ( buf+offset,
                celems(buf) - offset,
                L"\n\nPossible cause:\n\n" );

            offset = wcslen( buf );

            StringCchCatW ( buf+offset,
                celems(buf) - offset,
                lpSysMsg );

            LocalFree( (HLOCAL)lpSysMsg );
        }

        MessageBoxW( NULL,
                    buf,
                    L"Error",
                    MB_ICONERROR | MB_OK );
    }
    else {
        MessageBoxW( NULL,
                    buf,
                    APP_NAME,
                    MB_ICONINFORMATION | MB_OK );
    }

    return;
}

//
// Program entry point.
//

EXTERN_C int __cdecl wmain(int argc, WCHAR* argv[])
{
	HRESULT hr = S_OK;

	if ((argc == 3) && ((wcscmp(argv[1], L"-i") == 0) || (wcscmp(argv[1], L"/i") == 0)))
	{
		//
		// Try to install driver
		//
		LPWSTR  lpszPnpID;

		//
		// Inf file name specified, install the network component
		// from this file.
		//

		hr = GetPnpID(argv[2], &lpszPnpID);
		if (hr == S_OK)
		{
			hr = InstallSpecifiedComponent(argv[2], lpszPnpID, &GUID_DEVCLASS_NETSERVICE);
			CoTaskMemFree(lpszPnpID);
		} else
		{
			ErrMsg(hr, L"Error reading the INF file %s.", argv[2]);
		}
	} else if ((argc == 2) && ((wcscmp(argv[1], L"-u") == 0) || (wcscmp(argv[1], L"/u") == 0)))
	{
		//
		// Try to uninstall driver
		//
		hr = UninstallComponent(L"ls_openfw");
	} else
	{
		hr = HRESULT_FROM_WIN32(ERROR_INVALID_PARAMETER);
	}

	return hr;
}
